#ifndef BACKGROUND_H
#define BACKGROUND_H


///BGCNT defines ///
#define BG_MOSAIC_ENABLE		0x40
#define BG_COLOR_256			0x80
#define BG_COLOR_16				0x0

#define CharBaseBlock(n)		(((n)*0x4000)+0x6000000)
#define ScreenBaseBlock(n)		(((n)*0x800)+0x6000000)

#define CHAR_SHIFT				2
#define SCREEN_SHIFT			8
#define TEXTBG_SIZE_256x256		0x0
#define TEXTBG_SIZE_256x512		0x8000
#define TEXTBG_SIZE_512x256		0x4000
#define TEXTBG_SIZE_512x512		0xC000

#define ROTBG_SIZE_128x128		0x0
#define ROTBG_SIZE_256x256		0x4000
#define ROTBG_SIZE_512x512		0x8000
#define ROTBG_SIZE_1024x1024	0xC000

#define WRAPAROUND              0x1

// Define a class to take care of background management on the GBA
// specifically
class GBA_Background
{
	// Pointers to GBA Mem;
	// if text bg - [0] hoffset [1] yoffset
	// if otherwise - [0-3] PA-D [4-7] X/Y Highs and lows
	uint16* m_pBGCNT;	// pointer to background data
	uint16* m_pPOSDATA;	// pointer to text bg scroll data
	uint16* m_pROTDATA; // pointer to rot background orientation data
	uint8 m_nBg;	// which bg

	// background state
	int16 m_X, m_Y;
	int16 m_Rot, m_CentreX, m_CentreY;
	int32 m_Zoom;
	uint16 m_Base;

	// storage for copying position info
	int32 DX,DY;
	int16 PA,PB,PC,PD;

	void RecalcRot();

public:
	GBA_Background(uint8 nBg);	// we need to know which we are
	~GBA_Background();

	void Update();
	void CopyTileData(uint16* pSource, uint16 nCount);
	void CopyMapData(uint16* pSource, uint16 nCount);
	void CopyRow(uint16* pSource, uint8 sx, uint8 sy, uint8 nLen); 
	void CopyColumn(uint16* pSource, uint8 sx, uint8 sy, uint8 nLen); 

	// General methods
	bool IsEnabled();
	void Enable();
	void Disable();

	// Specific methods
	bool IsMosaic();
	void EnableMosaic();
	void DisableMosaic();

	// Set with low number (0-3)
	void SetSize(uint8 nSize);
	uint8 GetSize();

	// Tile and level map base get/set methods
	void SetTileSetBase(uint8 nBase);
	uint8 GetTileSetBase();
	void SetLevelMapBase(uint8 nBase);
	uint8 GetLevelMapBase();

	// Color Depth Get/Set Methods
	// Can be either defines or 0-1
	void SetColorDepth(uint8 nDepth);
	uint8 GetColorDepth();

	// Priority Get/Sets - (0-3)
	void SetPriority(uint8 nPri);
	uint8 GetPriority();

	// Interaction Functions
	void ScrollMap(int16 x, int16 y);
	void SetOffset(int16 x, int16 y);

	// Rotate is comulative
	void Rotate(int16 angle);
	// Get and set for rotation
	void SetRotation(int16 angle);
	int16 GetRotation();

	// Center get/sets
	void SetCenter(int16 x, int16 y);
	void GetCenter(int16* x, int16* y);
	// Zoom Get/Sets
	void SetZoom(int32 zoom);
	int32 GetZoom();
};

#define BPP

#endif
