#ifndef _INTEGER_H_
#define _INTEGER_H_

// portable extended boolean type
#include "bool.h"

// because everything includes this throw some extras here like
#include <string>
#ifdef String
#undef String
#endif
typedef std::string String;

/*!
Portable integer types. and basic operations
*/
typedef unsigned int	uint; // This is at least 16 bits

// Types that are the same on ALL platforms (like really even ps2 AND gba)
// 8 and 16 are the same
typedef signed char		int8;
typedef signed short	int16;
typedef unsigned char	uint8;
typedef unsigned short	uint16;
typedef unsigned char	byte;
typedef unsigned short	hword;

#ifndef __PS2
typedef signed long		int32;
typedef unsigned long	uint32;
	#ifndef __GBA
		#ifdef __LINUX
		typedef signed long long	int64;
		typedef unsigned long long	uint64;
		#else // _WIN32 || _MAC
		typedef signed __int64		int64;
		typedef unsigned __int64    uint64;
		#endif // _LINUX
	#endif // !__GBA
#endif // !__PS2

#ifdef __PS2
	#include <eekernel.h>
	typedef signed int		int32;
	typedef signed long		int64;
	typedef long128			int128;
	typedef unsigned int    uint32;
	typedef unsigned long   uint64;
	typedef u_long128       uint128;
#endif

/*
	Useful Defines
*/

// NEG_BITS don't do anything but they're here anyway.
#define NEG_BIT_8	0x80
#define NEG_BIT_16	0x8000
#define NEG_BIT_32	0x80000000
#define NEG_BIT_64	0x8000000000000000	// may not work

// Standard functions for common integer ops
#define BITS(x) (sizeof(x)<<3)	// sizeof is bytes, *8

// MAXSUZE Works with all unsigned integer types
template <class T>
inline T MAXSIZEU(T v) // for unsigned integers
{
	T mxsz = 0;
	// (variables can't really be more than 128 bits)
	uint8 x = BITS(T);
	while (x--)
		mxsz += 1<<x;
	return mxsz;
}

template <class T>
// MAXSUZE Works with all unsigned integer types except 64bit
inline T MAXSIZES(T& v) // for signed integers
{
	T mxsz = 0;
	// (variables can't really be more than 128 bits)
	uint8 x = BITS(T)-1;
	while (x--)
		mxsz += 1<<x;
	return mxsz;
}

#define MAX(a, b) (a > b ? a : b)

#define MIN(a, b) (a < b ? a : b)

#endif

