#include "gba.h"
#include "Sprite.h"
//#include "template_list.h"
#include "mem.h"

//List<int8, uint8> g_nFreeSprites;	// when we free a prite out of order
uint8 g_nMaxSprite = 0;	// highest sprite loaded

CSprite::CSprite() : m_pAttribs(0), m_nSprite(0xFF)
{
}
CSprite::~CSprite()
{
	if (m_pAttribs)
	{
		delete m_pAttribs;
		m_pAttribs->Clear();
	}

	m_pAttribs = 0;
	m_nSprite = 0xFF;
}

bool CSprite::LoadSprite(uint16 Offset, hword Size1, hword Shape, hword Color, uint8 Priority, uint16 Transparent)
{
	if (!m_pAttribs)
	{ // not data set, get some
		m_pAttribs = new CAttribs;
//		if (g_nFreeSprites.size())
//			(*m_pAttribs) = m_nSprite = g_nFreeSprites.pop_front();
//		else if (g_nMaxSprite < 128)
//			(*m_pAttribs) = m_nSprite = g_nMaxSprite++;
//		else
//			return FALSE; //  couldn't find a spot
	}
	else
		m_pAttribs->Clear();

	switch (Size1)
	{
	case SIZE_8:
		switch (Shape)
		{
		case TALL:	//1x2
		case WIDE:	//2x1
			m_nBlocksPerFrame = 2;
			break;
		default:	//1x1
			m_nBlocksPerFrame = 1;
			break;
		}
		break;
	case SIZE_16:	//always 2x2
		m_nBlocksPerFrame = 4;
		break;
	case SIZE_32:
		switch (Shape)
		{
		case TALL:	//2x4
		case WIDE:	//4x2
			m_nBlocksPerFrame = 8;
			break;
		default:	//4x4
			m_nBlocksPerFrame = 16;
			break;
		}
		break;
	case SIZE_64:
		switch (Shape)
		{
		case TALL:	//4x8
		case WIDE:	//8x4
			m_nBlocksPerFrame = 32;
			break;
		default:	//8x8
			m_nBlocksPerFrame = 64;
			break;
		}
		break;
	}
	if (Color == COLOR_256)
		m_nBlocksPerFrame *= 2;

	m_pAttribs->Attr0 |= Color;
	m_pAttribs->Attr0 |= Shape;
	m_pAttribs->Attr0 |= Transparent;
	m_pAttribs->Attr1 |= Size1;
	pStartOffset = Offset;
	m_pAttribs->SetCharacter(Offset);
	m_pAttribs->SetPriority(Priority);

	return TRUE;
}

void CSprite::DrawSprite()
{
	m_Anims.Update();
	//m_pAttribs->SetCharacter(pStartOffset + (m_Anims.GetCurFrame()*m_nBlocksPerFrame));

	CAttribs* pOAMMem = (CAttribs*)OAMmem;
	pOAMMem[0] = *m_pAttribs;
//	pOAMMem[0] = m_pAttribs->Attr0;
//	pOAMMem[1] = m_pAttribs->Attr1;
//	pOAMMem[2] = m_pAttribs->Attr2;
//	pOAMMem[3] = m_pAttribs->Attr3;
}

void CSprite::MoveSprite(int8 x, int8 y)
{
	m_pAttribs->SetX(m_pAttribs->GetX()+x);
	m_pAttribs->SetY(m_pAttribs->GetY()+y);
}
void CSprite::PutSprite(int16 x, int16 y)
{
	m_pAttribs->SetX(x);
	m_pAttribs->SetY(y);
}
void CSprite::InitAnims(uint8 Anims, uint8** pData, uint16* pDataSize, uint8 Delay)
{
	m_Anims.Init(Anims, pData, pDataSize, Delay);
}
void CSprite::DoAnim(uint8 nAnim)
{
	m_Anims.SetNextAnim(nAnim);
}

// Animation Class
CAnimation::CAnimation()
{
	m_nAnimations = 0;
	m_pData = 0;
	m_nDataSize = 0;
	m_nCurAnim = 0;
	m_nNextAnim = NOANIM;
	m_nCurFrame = 0;
	m_nDelay = m_nFrameTime = 0;
}
CAnimation::CAnimation(uint8 Anims, uint8** pData, uint16* pDataSize, uint8 Delay)
{
	Init(Anims, pData, pDataSize, Delay);
}
CAnimation::~CAnimation()
{
	if (m_pData)
		delete [] m_pData;
	if (m_nDataSize)
		delete [] m_nDataSize;
}

void CAnimation::Init(uint8 Anims, uint8** pData, uint16* pDataSize, uint8 Delay)
{
	if (!Anims)
		return;

	m_nAnimations = Anims;

	m_pData = new uint8*[Anims];
	for (int x = 0; x < Anims; x++)
		m_pData[x] = pData[x];
	//memcpy(m_pData, pData, Anims*2);
	m_nDataSize = new uint16[Anims];
	memcpy(m_nDataSize, pDataSize, Anims*2);

	m_nCurAnim = 0;
	m_nNextAnim = 0;
	m_nCurFrame = 0;

	m_nDelay = Delay;
}
void CAnimation::SetNextAnim(uint8 nAnim)
{
	if (nAnim >= m_nAnimations)
		m_nNextAnim = 0;
	else if (nAnim && nAnim != m_nCurAnim)
	{
		m_nCurAnim = nAnim;
		m_nCurFrame = 0;
		m_nNextAnim = 0;
	}
	else
		m_nNextAnim = nAnim;
}
void CAnimation::Update()
{
	m_nFrameTime++;
	if (m_nFrameTime < m_nDelay || !m_nAnimations)
		return;

	m_nFrameTime = 0;
	if (m_nCurFrame < (m_nDataSize[m_nCurAnim]>>10) - 1)
		m_nCurFrame++;
	else
	{
		m_nCurAnim = m_nNextAnim;
		m_nNextAnim = 0;
		m_nCurFrame = 0;
	}
	memcpy(OAMdata, m_pData[m_nCurAnim] + (m_nCurFrame<<10), 1024);
}
