#include "gba.h"
#include "TileMap.h"
#include "mem.h"
#include "GBA_Background.h"

bool layers[4] = {FALSE};

CTileMap::CTileMap(TileSet* pTileData, LevelMap* pMapData) : m_pBackground(0),
	m_MapData(*pMapData), m_TileData(*pTileData)
{
	m_nBigMapX = 0;
	m_nBigMapY = 0;
	m_nLtlMapX = 0;
	m_nLtlMapY = 0;
	m_nMapOffsetX = 0;
	m_nMapOffsetY = 0;
//	m_pTileRefCount = new uint16[m_TileData.nTiles];
//	zeromem(m_pTileRefCount, m_TileData.nTiles*2);
}
CTileMap::~CTileMap()
{
//	if (m_pTileRefCount)
//		delete [] m_pTileRefCount;

	if (m_pBackground)
		delete m_pBackground;
}

bool CTileMap::InitBg(uint16 sx, uint16 sy, uint8 nLayer)
{
	if (nLayer > 3)
		return ERROR;

	if (ITRUE(layers[nLayer]))
		return FALSE;

	layers[nLayer] = TRUE;
	m_pBackground = new GBA_Background(nLayer);
	m_pBackground->SetColorDepth(1);
	m_pBackground->SetSize(0);
	m_pBackground->Enable();
	m_pBackground->SetLevelMapBase(20 + nLayer*3);

	m_pBackground->CopyTileData((uint16*)m_TileData.pTiles,
								m_TileData.nTiles*64);
	m_nBigMapX = sx;
	m_nBigMapY = sy;
	m_nLtlMapX = 0;
	m_nLtlMapY = 0;
	m_nMapOffsetX = 0;
	m_nMapOffsetY = 0;
	uint16 nData[1024];
	
	int xcopy = (m_MapData.nX < 31 ? 30 : 31);
	int ycopy = (m_MapData.nY < 21 ? 20 : 21);

	for (int x = 0; x < xcopy; x++)
	{
		for (int y = 0; y < ycopy; y++)
		{
			nData[x + (y*32)] = m_MapData.pMap[(sx+x) + ((sy+y)*m_MapData.nX)];
		}
	}
	m_pBackground->CopyMapData((uint16*)&nData, 1024);
	return TRUE;
}
void CTileMap::CopyRow(int16 nDir)
{
}
void CTileMap::CopyColumn(int16 nDir)
{
	// move one column over
	if (nDir == -1)
	{
		// if +1 then we move out pointer over and copy a row on the right 
		// side of the screen (point+30)

		// move our point on our level map data
		if (m_nBigMapX + 1 >= m_MapData.nX)
			m_nBigMapX = 0;
		else
			m_nBigMapX += 1;

		// get the new column position of tilemap data to add
		uint16 nNewX = (m_nBigMapX + 30) % m_MapData.nX;
		uint16 startpos = nNewX + (m_nBigMapY*m_MapData.nX);
		uint16* pMapPos = m_MapData.pMap + startpos;

		// copy info into temp variable
		uint16 pNewData[20];
		for (int loop = 0; loop < 20; loop++)
		{
			pNewData[loop] = *pMapPos;
			pMapPos += m_MapData.nX;
		}

		// move out point of the level map in memory
		if (m_nLtlMapX + 1 >= 32)
			m_nLtlMapX = 0;
		else
			m_nLtlMapX += 1;

		// get the column on screen to update
		uint8 nx = (m_nLtlMapX + 30) % 32;
		uint8 ny = m_nLtlMapY;
		m_pBackground->CopyColumn(pNewData, nx, ny, 20);
	}
	else if (nDir == 1)
	{
		// if -1 then we move out pointer over and copy a row on the left 
		// side of the screen (point)

		// move our point on our level map data
		if ((int32)m_nBigMapX - 1 < 0)
			m_nBigMapX = m_MapData.nX - 1;
		else
			m_nBigMapX -= 1;

		// get the new column position of tilemap data to add
		uint16 nNewX = m_nBigMapX;
		uint16 startpos = nNewX + (m_nBigMapY*m_MapData.nX);
		uint16* pMapPos = m_MapData.pMap + startpos;

		// copy info into temp variable
		uint16 pNewData[20];
		for (int loop = 0; loop < 20; loop++)
		{
			pNewData[loop] = *pMapPos;
			pMapPos += m_MapData.nX;
		}

		// move out point of the level map in memory
		if ((int32)m_nLtlMapX - 1 < 0)
			m_nLtlMapX = 31;
		else
			m_nLtlMapX -= 1;

		// get the column on screen to update
		uint8 nx = m_nLtlMapX;
		uint8 ny = m_nLtlMapY;
		m_pBackground->CopyColumn(pNewData, nx, ny, 20);
	}
}
void CTileMap::MoveBg(int16 mx, int16 my)
{
	// X first
	// scroll out little map and see if we need to add a new column
	int16 newX = m_nMapOffsetX + mx;
	int16 moveBig = (m_nMapOffsetX/8) - (newX/8 + (newX < 0 ? -1 : 0));
	int16 moveMod = (moveBig > 0 ? 1 : -1);
	m_nMapOffsetX = newX;
	if (m_nMapOffsetX < 0)
		m_nMapOffsetX += 256;

	while (moveBig)
	{
		moveBig -= moveMod;
		CopyColumn(moveMod);
	}

/*	int16 toscroll = (m_nLtlMapX%8)-mx;
	if (toscroll < 0)
		toscroll -= 8;
	toscroll /= 8;


	m_nLtlMapX += mx;
	if (m_nLtlMapX > 255)
		m_nLtlMapX -= 256;
	else if (m_nLtlMapX < 0)
		m_nLtlMapX += 256;

	if (toscroll)
	{
		int16 mod = (toscroll > 0 ? -1 : 1);
		while (toscroll != 0)
		{
			// move all data one tile
			if ((int32)m_nBigMapX + mod < 0)
				m_nBigMapX = m_MapData.nX;
			else if ((int32)m_nBigMapX + mod > m_MapData.nX)
				m_nBigMapX = 0;
			else
				m_nBigMapX+=mod;

			toscroll += mod;

			// get the new column in data to add
			uint16 nNewX = m_nBigMapX + (mod > 0 ? 29 : 0) % 32;
			uint16 pNewData[20];
			uint16 startpos = nNewX + (m_nBigMapY*m_MapData.nX);
			uint16* pMapPos = m_MapData.pMap + startpos;
			for (int loop = 0; loop < 20; loop++)
			{
				pNewData[loop] = *pMapPos;
				pMapPos += m_MapData.nX;
			}

			// get the new column on the map to add
			uint8 nx = ((m_nLtlMapX/8) + (mod<0 ? 29 : -1)) % 32;
			uint8 ny = (m_nLtlMapY/8);
			m_pBackground->CopyColumn(pNewData, nx, ny, 20);
		}
	}*/
}
void CTileMap::DrawBg()
{
	m_pBackground->SetOffset(m_nMapOffsetX, m_nMapOffsetY);
	m_pBackground->Update();
}
